package samples.files;

/*
 * A simple application that demonstrates the use of Aspera Java fasp Manager
 * API. Aspera, Inc.
 */

import com.asperasoft.faspmanager.Encryption;
import com.asperasoft.faspmanager.FaspManager;
import com.asperasoft.faspmanager.FaspManagerException;
import com.asperasoft.faspmanager.LocalLocation;
import com.asperasoft.faspmanager.Policy;
import com.asperasoft.faspmanager.RemoteLocation;
import com.asperasoft.faspmanager.Resume;
import com.asperasoft.faspmanager.TransferOrder;
import com.asperasoft.faspmanager.XferParams;
import com.google.gson.JsonParser;

class SimpleFileTransfer {

	/*
	 * An object of this class can be set as the authorizer. Once set, the authorizationRequest method is invoked if the Aspera server
	 * is configured to use authorization. This does not apply when using fasp manager as a client.
	 */
	// static class FaspManagerAuthorizer implements SessionAuthorizer {
	// public synchronized boolean authorizationRequest(SessionStats fts) {
	// return true;
	// }
	// }

	public static void main(String[] args) throws Exception {

		try {

			/*
			 * Configure the fasp Manager. The fasp manager can be configured using the setter methods provided by Environment class.
			 * Depending on your deployment environment you need to set some or all of those configuration attributes. Refer to the
			 * documentation bundled with the SDK for more details.
			 */

			/* Set path to ascp if Fasp Manager is unable to find it automatically or to override. */
			// Environment.setFasp2ScpPath("C:\\Program Files (x86)\\Aspera\\Enterprise Server\\bin\\ascp.exe");

			/*
			 * Change to true to enable server side transfer monitoring. Use along with a Global Listener to receive notifications
			 */
			FaspManager.getSingleton().listenForServerSessions(false);

			/*
			 * Use this in conjunction with aspera.conf setting to enable authorization
			 */
			// FaspManager.getSingleton().setAuthorizer(myAuthorizer);

			/*
			 * A listener added to the fasp Manager(as below) will be notified on all events and on all jobs. It is also possible to
			 * add a listener to only a particular job of interest.
			 */
			FaspManagerListener faspManagerListener = new FaspManagerListener();
			FaspManager.getSingleton().addListener(faspManagerListener);

			/*
			 * Start a download. Make sure the destination directory exists (in the example below the destination directory is ".").
			 */
			startDownload("demo.asperasoft.com", "aspera", "demoaspera", new String[] { "aspera-test-dir-small/10MB.1" }, "./âspëra-10MB");

			/*
			 * Wait for the download to finish, then set up an upload job. At this point, the source specified should actually exist.
			 */
			while (faspManagerListener.getFileTransferCount() < 1)
				Thread.sleep(100);
			System.out.println("Download complete, starting upload:");
			startUpload("demo.asperasoft.com", "asperaweb", "demoaspera", new String[] { "âspëra-10MB" }, "Upload");

			/*
			 * Fasp Manager needs to be stopped before exiting the application. So, wait for transfers to complete before continuing:
			 */
			while (faspManagerListener.getFileTransferCount() < 2)
				Thread.sleep(100);

			FaspManager.destroy();

		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	/*
	 * Set the transfer parameters for this transfer. Ideally, the target rate is equal to the bandwidth capacity of the Aspera
	 * computer (in kilo bits per second). For example, for a T1 link set targetRate = 1500 (1.5 mega bits per second). fasp will never
	 * exceed the specified target rate.
	 */
	static XferParams myTransferParams() {
		XferParams p = new XferParams();
		/* For most cases, these port numbers need not be changed */
		p.tcpPort = 22;
		p.udpPort = 33001;
		p.encryption = Encryption.NONE;
		// Policy.ADAPTIVE: fasp will try to match the target rate while being
		// fair to other traffic on the network
		p.policy = Policy.FAIR;
		// p.cookie = "FaspManagerSample1-Cookie";

		// Sample tags
		p.tags = new JsonParser().parse("{\"my-company\": {\"toto\":123}}").getAsJsonObject();

		// With newer versions of ascp if a token is passed it must be valid
		// regardles of whether or not username/password authentication is used
		// p.token = "FaspManagerSample1-Token";
		// Resume.OFF will not resume the transfer after a broken transfer
		p.resumeCheck = Resume.OFF;
		p.preCalculateJobSize = true;
		return p;
	}

	/*
	 * Sets up and submits a download job.
	 */
	static String startDownload(String host, String user, String password, String[] remoteSourcePaths, String localDestinationPath)
			throws FaspManagerException {

		/* Create a remote source location object */
		RemoteLocation remoteSourceFiles = new RemoteLocation(host, user, password);

		/* Add paths on the remote source to the source object */
		for (String path : remoteSourcePaths)
			remoteSourceFiles.addPath(path);

		/* Create a local destination object */
		LocalLocation localDestination = new LocalLocation();

		/* Add path to the destination object */
		localDestination.addPath(localDestinationPath);

		/*
		 * Setup a job order. A job order consists of source, destination and the transfer params(XferParams object)
		 */
		TransferOrder order = new TransferOrder(remoteSourceFiles, localDestination, myTransferParams());

		/* Submit the job for transfer */
		return FaspManager.getSingleton().startTransfer(order);
	}

	/*
	 * Sets up and submits an upload job.
	 */
	static String startUpload(String host, String user, String password, String[] localSourcePaths, String remoteDestinationPath)
			throws FaspManagerException {

		/* Create a local source location object */
		LocalLocation localSourceFiles = new LocalLocation();

		/* Add local paths to the source object */
		for (String path : localSourcePaths)
			localSourceFiles.addPath(path);

		/* Create a remote destination object */
		RemoteLocation remoteDestination = new RemoteLocation(host, user, password);

		/* Add remote path to the destination object */
		remoteDestination.addPath(remoteDestinationPath);

		/*
		 * Setup a job order. A job order consists of source, destination and the transfer params(XferParams object)
		 */
		TransferOrder order = new TransferOrder(localSourceFiles, remoteDestination, myTransferParams());

		/* Submit the job for transfer */
		return FaspManager.getSingleton().startTransfer(order);
	}

}
